using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Runtime.InteropServices;

using System.Windows.Forms;


using System.Drawing.Imaging;

namespace ScreenshotCaptureWithMouse.ScreenCapture
{
	class CaptureScreen
	{
		static Bitmap CaptureDesktop(Rectangle Monitor){
			
			IntPtr hdlDesktopWindow = Win32.User.GetDesktopWindow();
			IntPtr hdlSourceDc = Win32.User.GetWindowDC(hdlDesktopWindow);
			IntPtr hDest = Win32.Gdi.CreateCompatibleDC(hdlSourceDc);
			IntPtr hBmp = Win32.Gdi.CreateCompatibleBitmap(hdlSourceDc, Monitor.Width, Monitor.Height);
			IntPtr hOldBmp = Win32.Gdi.SelectObject(hDest, hBmp);
			Win32.Gdi.BitBlt(hDest, 0, 0, Monitor.Width, Monitor.Height, hdlSourceDc, Monitor.X, Monitor.Y, CopyPixelOperation.SourceCopy );//| CopyPixelOperation.CaptureBlt);
			Bitmap bmp = Bitmap.FromHbitmap(hBmp);
			IntPtr g=Win32.Gdi.DeleteObject(hOldBmp);
			IntPtr b=Win32.Gdi.DeleteObject(hBmp);
			IntPtr c=Win32.Gdi.DeleteDC(hDest);
			IntPtr f=Win32.Gdi.DeleteDC(hdlSourceDc);

			return bmp;
		}
		
		static Bitmap CaptureCursorOld(Rectangle Monitor,ref int x, ref int y){
			Bitmap bmp=null;
			IntPtr hicon;
			Win32Stuff.CURSORINFO ci = new Win32Stuff.CURSORINFO();
			Win32Stuff.ICONINFO icInfo;
			ci.cbSize = Marshal.SizeOf(ci);
			if (Win32Stuff.GetCursorInfo(out ci))
			{
				if (ci.flags == Win32Stuff.CURSOR_SHOWING)
				{
					hicon = Win32Stuff.CopyIcon(ci.hCursor);
					if (Win32Stuff.GetIconInfo(hicon, out icInfo))
					{
						x = ci.ptScreenPos.x - ((int)icInfo.xHotspot) - Monitor.Left;
						y = ci.ptScreenPos.y - ((int)icInfo.yHotspot) - Monitor.Top;
						
						try{
							Icon ic = Icon.FromHandle(hicon);
							bmp = ic.ToBitmap();
						}
						catch(Exception){}
						return bmp;
					}
				}
			}
			return null;
		}
		
		static Bitmap CaptureCursor(Rectangle Monitor,ref int x, ref int y, ref int spotx, ref int spoty){
			Win32Stuff.CURSORINFO cursorInfo = new Win32Stuff.CURSORINFO();
			cursorInfo.cbSize = Marshal.SizeOf(cursorInfo);
			if (!Win32Stuff.GetCursorInfo(out cursorInfo)) return null;
			if (cursorInfo.flags != Win32Stuff.CURSOR_SHOWING) return null;

			IntPtr hicon = Win32Stuff.CopyIcon(cursorInfo.hCursor);
			Win32Stuff.ICONINFO iconInfo;
			IntPtr Mask;
			if ((hicon != IntPtr.Zero)&&(Win32Stuff.GetIconInfo(hicon, out iconInfo))){
				
				spotx=((int)iconInfo.xHotspot);
				spoty=((int)iconInfo.yHotspot);
				x = cursorInfo.ptScreenPos.x - spotx - Monitor.Left;
				y = cursorInfo.ptScreenPos.y - spoty - Monitor.Top;
				Mask=iconInfo.hbmMask;
				if (iconInfo.hbmColor  != IntPtr.Zero) Win32.Gdi.DeleteObject(iconInfo.hbmColor);
				if (iconInfo.hbmMask != IntPtr.Zero) Win32.Gdi.DeleteObject(iconInfo.hbmMask);
			}
			else{
				spotx=0;
				spoty=0;
				x = cursorInfo.ptScreenPos.x - Monitor.Left;
				y = cursorInfo.ptScreenPos.y - Monitor.Top;
				Mask=IntPtr.Zero;
			}

			try{
				Bitmap bmp = Icon.FromHandle(hicon).ToBitmap();
				Win32Stuff.DestroyIcon(hicon);
				return bmp;
			}catch(Exception){}
			
			try{
				using (Bitmap maskBitmap = Bitmap.FromHbitmap(Mask))
				{
					// Is this a monochrome cursor?
					if (maskBitmap.Height == maskBitmap.Width * 2)
					{
						Bitmap resultBitmap = new Bitmap(maskBitmap.Width, maskBitmap.Width);

						Graphics desktopGraphics = Graphics.FromHwnd(Win32Stuff.GetDesktopWindow());
						IntPtr desktopHdc = desktopGraphics.GetHdc();

						IntPtr maskHdc = Win32.Gdi.CreateCompatibleDC(desktopHdc);
						IntPtr oldPtr = Win32.Gdi.SelectObject(maskHdc, maskBitmap.GetHbitmap());

						using (Graphics resultGraphics = Graphics.FromImage(resultBitmap))
						{
							IntPtr resultHdc = resultGraphics.GetHdc();

							// These two operation will result in a black cursor over a white background.
							// Later in the code, a call to MakeTransparent() will get rid of the white background.
							Win32.Gdi.BitBlt(resultHdc, 0, 0, 32, 32, maskHdc, 0, 32, CopyPixelOperation.SourceCopy);
							Win32.Gdi.BitBlt(resultHdc, 0, 0, 32, 32, maskHdc, 0, 0, CopyPixelOperation.SourceInvert);

							resultGraphics.ReleaseHdc(resultHdc);
						}

						IntPtr newPtr = Win32.Gdi.SelectObject(maskHdc, oldPtr);
						Win32.Gdi.DeleteDC(newPtr);
						Win32.Gdi.DeleteDC(maskHdc);
						desktopGraphics.ReleaseHdc(desktopHdc);

						// Remove the white background from the BitBlt calls,
						// resulting in a black cursor over a transparent background.
						resultBitmap.MakeTransparent(Color.White);
						return resultBitmap;
					}
				}
			}
			catch{
			}
			return null;
		}

		public static bool CaptureDesktopWithCursor(Rectangle Monitor,ref Bitmap desktopBMP,bool currentIntersect){
			int cursorX=0;
			int cursorY=0;
			int spotX=0;
			int spotY=0;
			bool intersect=currentIntersect;

			desktopBMP = CaptureDesktop(Monitor);
			Bitmap cursorBMP = CaptureCursor(Monitor, ref cursorX, ref cursorY,ref spotX, ref spotY);
			if (desktopBMP != null){
				Rectangle r1 = new Rectangle(cursorX+spotX, cursorY+spotY, 1,1);
				Rectangle r2= new Rectangle(0,0,desktopBMP.Width,desktopBMP.Height);
				Graphics g = Graphics.FromImage(desktopBMP);
				if (r1.IntersectsWith(r2)&&((cursorX!=0)||(cursorY!=0))){
					intersect=true;
					if (cursorBMP != null){
						Rectangle r = new Rectangle(cursorX, cursorY, cursorBMP.Width, cursorBMP.Height);
						g.DrawImage(cursorBMP, r);
						r.Offset(spotX-cursorBMP.Width/2,spotY-cursorBMP.Height/2);
						g.DrawRectangle(new Pen(Color.Red,3),r);
						cursorBMP.Dispose();
					}
				}
				else{
					intersect=false;
					g.DrawRectangle(new Pen(Color.Red,3),new Rectangle(cursorX,cursorY,20,20));
				}
				g.Flush();
				g.Dispose();
			}
			return intersect;
		}
	}
}
