using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;

namespace ScreenshotCaptureWithMouse.ScreenCapture
{
	
	namespace Win32
	{
		/// <summary>
		/// Provides access to native function from gdi32.dll.
		/// </summary>
		public static class Gdi
		{
			#region fields
			#endregion

			#region Class Variables
			public const int SRCCOPY = 13369376;
			public const int SRCINVERT = 0x660046;
			#endregion
			
			
			#region ctor

			/// <summary>
			/// Initializes static members of Gdi class.
			/// </summary>
			static Gdi()
			{
			}

			#endregion

			#region properties
			#endregion

			#region gdi32.dll native methods

			/// <summary>
			/// This function transfers pixels from a specified source rectangle to a specified destination
			/// rectangle, altering the pixels according to the selected raster operation (ROP) code.
			/// </summary>
			/// <param name="hdcDest">Handle to the destination device context.</param>
			/// <param name="nXDest">Specifies the logical x-coordinate of the upper-left corner of the destination rectangle.</param>
			/// <param name="nYDest">Specifies the logical y-coordinate of the upper-left corner of the destination rectangle.</param>
			/// <param name="nWidth">Specifies the logical width of the source and destination rectangles.</param>
			/// <param name="nHeight">Specifies the logical height of the source and the destination rectangles.</param>
			/// <param name="hdcSrc">Handle to the source device context.</param>
			/// <param name="nXSrc">Specifies the logical x-coordinate of the upper-left corner of the source rectangle.</param>
			/// <param name="nYSrc">Specifies the logical y-coordinate of the upper-left corner of the source rectangle. </param>
			/// <param name="dwRop">Specifies a raster-operation code. These codes define how the color data
			/// for the source rectangle is to be combined with the color data for the destination rectangle
			/// to achieve the final color.</param>
			/// <returns>Nonzero indicates success. Zero indicates failure. To get extended error information, call GetLastError.</returns>
			[DllImport("gdi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern bool BitBlt(IntPtr hdcDest, int nXDest, int nYDest, int nWidth, int nHeight, IntPtr hdcSrc, int nXSrc, int nYSrc, CopyPixelOperation dwRop);

			/// <summary>
			/// The ReleaseDC function releases a device context (DC), freeing it for use by other applications.
			/// The effect of the ReleaseDC function depends on the type of DC. It frees only common and
			/// window DCs. It has no effect on class or private DCs.
			/// </summary>
			/// <param name="hWnd">A handle to the window whose DC is to be released.</param>
			/// <param name="hDc">A handle to the DC to be released.</param>
			/// <returns>The return value indicates whether the DC was released. If the DC was released,
			/// the return value is 1. If the DC was not released, the return value is zero.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern bool ReleaseDC(IntPtr hWnd, IntPtr hDc);

			/// <summary>
			/// The DeleteDC function deletes the specified device context (DC).
			/// </summary>
			/// <param name="hDc">A handle to the device context.</param>
			/// <returns>If the function succeeds, the return value is nonzero. If the function fails,
			/// the return value is zero.</returns>
			[DllImport("gdi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr DeleteDC(IntPtr hDc);

			/// <summary>
			/// This function deletes a logical pen, brush, font, bitmap, region, or palette, freeing
			/// all system resources associated with the object. After the object is deleted,
			/// the specified handle is no longer valid.
			/// </summary>
			/// <param name="hObject">Handle to a logical pen, brush, font, bitmap, region, or palette.</param>
			/// <returns>Nonzero indicates success. Zero indicates that the specified handle is not valid
			/// or that the handle is currently selected into a device context. To get extended error
			/// information, call GetLastError.</returns>
			[DllImport("gdi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr DeleteObject(IntPtr hObject);

			/// <summary>
			/// This function selects an object into a specified device context. The new object
			/// replaces the previous object of the same type.
			/// </summary>
			/// <param name="hdc">Handle to the device context.</param>
			/// <param name="hgdiobj">Handle to the object to be selected.</param>
			/// <returns>If the selected object is not a region, the handle of the object being replaced indicates success.</returns>
			[DllImport("gdi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr SelectObject(IntPtr hdc, IntPtr hgdiobj);

			/// <summary>
			/// This function creates a memory device context (DC) compatible with the specified device.
			/// </summary>
			/// <param name="hdc">Handle to an existing device context. If this handle is NULL, the
			/// function creates a memory device context compatible with the application's current screen.
			/// </param>
			/// <param name="nWidth">Specifies the bitmap width, in pixels.</param>
			/// <param name="nHeight">Specifies the bitmap height, in pixels.</param>
			/// <returns>A handle to the bitmap indicates success. NULL indicates failure. To get extended
			/// error information, call GetLastError.</returns>
			[DllImport("gdi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr CreateCompatibleBitmap(IntPtr hdc, int nWidth, int nHeight);

			/// <summary>
			/// This function creates a memory device context (DC) compatible with the specified device.
			/// </summary>
			/// <param name="hdc">Handle to an existing device context. If this handle is NULL, the
			/// function creates a memory device context compatible with the application's current
			/// screen.</param>
			/// <returns>The handle to a memory device context indicates success. NULL indicates failure.
			/// To get extended error information, call GetLastError.</returns>
			[DllImport("gdi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr CreateCompatibleDC(IntPtr hdc);

			#endregion

			#region public members
			#endregion
		}
	}
}
