using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;



/// <summary>
/// Provides access to the Win32 API.
/// </summary>

namespace ScreenshotCaptureWithMouse.ScreenCapture
{
	namespace Win32
	{
		/// <summary>
		/// Provides access to native functions from user32.dll.
		/// </summary>
		public sealed class User
		{
			#region constants

			/// <summary>
			/// The class name of the console.
			/// </summary>
			private const String ConsoleClassName = "ConsoleWindowClass";

			/*
      /// <summary>
      /// flags for show window async.
      /// </summary>
      private const int WS_SHOWNORMAL = 1;
			 */

			#endregion

			#region Win32 message id and keys

			///summary>
			/// Virtual Messages.
			/// </summary>
			public enum WMessages : int
			{
				WM_LBUTTONDOWN = 0x201, //Left mousebutton down
				WM_LBUTTONUP = 0x202,  //Left mousebutton up
				WM_LBUTTONDBLCLK = 0x203, //Left mousebutton doubleclick
				WM_RBUTTONDOWN = 0x204, //Right mousebutton down
				WM_RBUTTONUP = 0x205,   //Right mousebutton up
				WM_RBUTTONDBLCLK = 0x206, //Right mousebutton doubleclick
				WM_KEYDOWN = 0x100,  //Key down
				WM_KEYUP = 0x101,   //Key up
				WM_CHAR = 0x102
			}

			/// <summary>
			/// Virtual Keys.
			/// </summary>
			public enum VKeys : int
			{
				VK_LBUTTON = 0x01,   //Left mouse button
				VK_RBUTTON = 0x02,   //Right mouse button
				VK_CANCEL = 0x03,   //Control-break processing
				VK_MBUTTON = 0x04,   //Middle mouse button (three-button mouse)
				VK_BACK = 0x08,   //BACKSPACE key
				VK_TAB = 0x09,   //TAB key
				VK_CLEAR = 0x0C,   //CLEAR key
				VK_RETURN = 0x0D,   //ENTER key
				VK_SHIFT = 0x10,   //SHIFT key
				VK_CONTROL = 0x11,   //CTRL key
				VK_MENU = 0x12,   //ALT key
				VK_PAUSE = 0x13,   //PAUSE key
				VK_CAPITAL = 0x14,   //CAPS LOCK key
				VK_ESCAPE = 0x1B,   //ESC key
				VK_SPACE = 0x20,   //SPACEBAR
				VK_PRIOR = 0x21,   //PAGE UP key
				VK_NEXT = 0x22,   //PAGE DOWN key
				VK_END = 0x23,   //END key
				VK_HOME = 0x24,   //HOME key
				VK_LEFT = 0x25,   //LEFT ARROW key
				VK_UP = 0x26,   //UP ARROW key
				VK_RIGHT = 0x27,   //RIGHT ARROW key
				VK_DOWN = 0x28,   //DOWN ARROW key
				VK_SELECT = 0x29,   //SELECT key
				VK_PRINT = 0x2A,   //PRINT key
				VK_EXECUTE = 0x2B,   //EXECUTE key
				VK_SNAPSHOT = 0x2C,   //PRINT SCREEN key
				VK_INSERT = 0x2D,   //INS key
				VK_DELETE = 0x2E,   //DEL key
				VK_HELP = 0x2F,   //HELP key
				VK_0 = 0x30,   // 0 key
				VK_1 = 0x31,   // 1 key
				VK_2 = 0x32,   // 2 key
				VK_3 = 0x33,   // 3 key
				VK_4 = 0x34,   // 4 key
				VK_5 = 0x35,   // 5 key
				VK_6 = 0x36,   // 6 key
				VK_7 = 0x37,   // 7 key
				VK_8 = 0x38,   // 8 key
				VK_9 = 0x39,   // 9 key
				VK_A = 0x41,   // A key
				VK_B = 0x42,   // B key
				VK_C = 0x43,   // C key
				VK_D = 0x44,   // D key
				VK_E = 0x45,   // E key
				VK_F = 0x46,   // F key
				VK_G = 0x47,   // G key
				VK_H = 0x48,   // H key
				VK_I = 0x49,   // I key
				VK_J = 0x4A,   //J key
				VK_K = 0x4B,   //K key
				VK_L = 0x4C,   //L key
				VK_M = 0x4D,   //M key
				VK_N = 0x4E,    //N key
				VK_O = 0x4F,   //O key
				VK_P = 0x50,    //P key
				VK_Q = 0x51,   //Q key
				VK_R = 0x52,   //R key
				VK_S = 0x53,   //S key
				VK_T = 0x54,   //T key
				VK_U = 0x55,   //U key
				VK_V = 0x56,   //V key
				VK_W = 0x57,   //W key
				VK_X = 0x58,   //X key
				VK_Y = 0x59,   //Y key
				VK_Z = 0x5A,    //Z key
				VK_NUMPAD0 = 0x60,   //Numeric keypad 0 key
				VK_NUMPAD1 = 0x61,   //Numeric keypad 1 key
				VK_NUMPAD2 = 0x62,   //Numeric keypad 2 key
				VK_NUMPAD3 = 0x63,   //Numeric keypad 3 key
				VK_NUMPAD4 = 0x64,   //Numeric keypad 4 key
				VK_NUMPAD5 = 0x65,   //Numeric keypad 5 key
				VK_NUMPAD6 = 0x66,   //Numeric keypad 6 key
				VK_NUMPAD7 = 0x67,   //Numeric keypad 7 key
				VK_NUMPAD8 = 0x68,   //Numeric keypad 8 key
				VK_NUMPAD9 = 0x69,   //Numeric keypad 9 key
				VK_MULTIPLY = 0x6B, // Multiply key
				VK_ADD = 0x6B, // Add key
				VK_SEPARATOR = 0x6C,   //Separator key
				VK_SUBTRACT = 0x6D,   //Subtract key
				VK_DECIMAL = 0x6E,   //Decimal key
				VK_DIVIDE = 0x6F,   //Divide key
				VK_F1 = 0x70,   //F1 key
				VK_F2 = 0x71,   //F2 key
				VK_F3 = 0x72,   //F3 key
				VK_F4 = 0x73,   //F4 key
				VK_F5 = 0x74,   //F5 key
				VK_F6 = 0x75,   //F6 key
				VK_F7 = 0x76,   //F7 key
				VK_F8 = 0x77,   //F8 key
				VK_F9 = 0x78,   //F9 key
				VK_F10 = 0x79,   //F10 key
				VK_F11 = 0x7A,   //F11 key
				VK_F12 = 0x7B,   //F12 key
				VK_SCROLL = 0x91,   //SCROLL LOCK key
				VK_LSHIFT = 0xA0,   //Left SHIFT key
				VK_RSHIFT = 0xA1,   //Right SHIFT key
				VK_LCONTROL = 0xA2,   //Left CONTROL key
				VK_RCONTROL = 0xA3,    //Right CONTROL key
				VK_LMENU = 0xA4,      //Left MENU key
				VK_RMENU = 0xA5,   //Right MENU key
				VK_PLAY = 0xFA,   //Play key
				VK_ZOOM = 0xFB, //Zoom key
			}

			/// <summary>
			/// Mapping table for key strings to VKeys.
			/// </summary>
			public static Dictionary<string, VKeys> Keys =new Dictionary<string, VKeys>()
			{
				{"{LBUTTON}", VKeys.VK_LBUTTON},
				{"{RBUTTON}", VKeys.VK_RBUTTON},
				{"{CANCEL}",  VKeys.VK_CANCEL}, // Ctrl-Break
				{"{MBUTTON}", VKeys.VK_MBUTTON},
				{"{BACK}", VKeys.VK_BACK},
				{"{TAB}", VKeys.VK_TAB},
				{"{CLEAR}", VKeys.VK_CLEAR},
				{"{RETURN}", VKeys.VK_RETURN},
				{"{ENTER}", VKeys.VK_RETURN},
				{"{SHIFT}", VKeys.VK_SHIFT},
				{"{CONTROL}", VKeys.VK_CONTROL},
				{"{ALT}", VKeys.VK_MENU},
				{"{MENU}", VKeys.VK_MENU},
				{"{PAUSE}", VKeys.VK_PAUSE},
				{"{CAPITAL}", VKeys.VK_CAPITAL},
				{"{ESCAPE}", VKeys.VK_ESCAPE},
				{"{SPACE}", VKeys.VK_SPACE},
				{" ", VKeys.VK_SPACE},
				{"{PRIOR}", VKeys.VK_PRIOR},
				{"{NEXT}", VKeys.VK_NEXT},
				{"{END}", VKeys.VK_END},
				{"{HOME}", VKeys.VK_HOME},
				{"{LEFT}", VKeys.VK_LEFT},
				{"{UP}", VKeys.VK_UP},
				{"{RIGHT}", VKeys.VK_RIGHT},
				{"{DOWN}", VKeys.VK_DOWN},
				{"{SELECT}", VKeys.VK_SELECT},
				{"{PRINT}", VKeys.VK_PRINT},
				{"{EXECUTE}", VKeys.VK_EXECUTE},
				{"{SNAPSHOT}", VKeys.VK_SNAPSHOT},
				{"{INSERT}", VKeys.VK_INSERT},
				{"{DELETE}", VKeys.VK_DELETE},
				{"{HELP}", VKeys.VK_HELP},
				{"0", VKeys.VK_0},
				{"1", VKeys.VK_1},
				{"2", VKeys.VK_2},
				{"3", VKeys.VK_3},
				{"4", VKeys.VK_4},
				{"5", VKeys.VK_5},
				{"6", VKeys.VK_6},
				{"7", VKeys.VK_7},
				{"8", VKeys.VK_8},
				{"9", VKeys.VK_9},
				{"A", VKeys.VK_A},
				{"B", VKeys.VK_B},
				{"C", VKeys.VK_C},
				{"D", VKeys.VK_D},
				{"E", VKeys.VK_E},
				{"F", VKeys.VK_F},
				{"G", VKeys.VK_G},
				{"H", VKeys.VK_H},
				{"I", VKeys.VK_I},
				{"J", VKeys.VK_J},
				{"K", VKeys.VK_K},
				{"L", VKeys.VK_L},
				{"M", VKeys.VK_M},
				{"N", VKeys.VK_N},
				{"O", VKeys.VK_O},
				{"P", VKeys.VK_P},
				{"Q", VKeys.VK_Q},
				{"R", VKeys.VK_R},
				{"S", VKeys.VK_S},
				{"T", VKeys.VK_T},
				{"U", VKeys.VK_U},
				{"V", VKeys.VK_V},
				{"W", VKeys.VK_W},
				{"X", VKeys.VK_X},
				{"Y", VKeys.VK_Y},
				{"Z", VKeys.VK_Z},
				{"{NUMPAD0}", VKeys.VK_NUMPAD0},
				{"{NUMPAD1}", VKeys.VK_NUMPAD1},
				{"{NUMPAD2}", VKeys.VK_NUMPAD2},
				{"{NUMPAD3}", VKeys.VK_NUMPAD3},
				{"{NUMPAD4}", VKeys.VK_NUMPAD4},
				{"{NUMPAD5}", VKeys.VK_NUMPAD5},
				{"{NUMPAD6}", VKeys.VK_NUMPAD6},
				{"{NUMPAD7}", VKeys.VK_NUMPAD7},
				{"{NUMPAD8}", VKeys.VK_NUMPAD8},
				{"{NUMPAD9}", VKeys.VK_NUMPAD9},
				{"{SEPARATOR}", VKeys.VK_SEPARATOR},
				{"{SUBTRACT}", VKeys.VK_SUBTRACT},
				{"{ADD}", VKeys.VK_ADD},
				{"{MULTIPLY}", VKeys.VK_MULTIPLY},
				{"{DECIMAL}", VKeys.VK_DECIMAL},
				{"{DIVIDE}", VKeys.VK_DIVIDE},
				{",", VKeys.VK_DECIMAL},
				{"-", VKeys.VK_SUBTRACT},
				{"+", VKeys.VK_ADD},
				{"*", VKeys.VK_MULTIPLY},
				{"/", VKeys.VK_DIVIDE},
				{"{F1}", VKeys.VK_F1},
				{"{F2}", VKeys.VK_F2},
				{"{F3}", VKeys.VK_F3},
				{"{F4}", VKeys.VK_F4},
				{"{F5}", VKeys.VK_F5},
				{"{F6}", VKeys.VK_F6},
				{"{F7}", VKeys.VK_F7},
				{"{F8}", VKeys.VK_F8},
				{"{F9}", VKeys.VK_F9},
				{"{F10}", VKeys.VK_F10},
				{"{F11}", VKeys.VK_F11},
				{"{F12}", VKeys.VK_F12},
				{"{SCROLL}", VKeys.VK_SCROLL},
				{"{LSHIFT}", VKeys.VK_LSHIFT},
				{"{RSHIFT}", VKeys.VK_RSHIFT},
				{"{LCONTROL}", VKeys.VK_LCONTROL},
				{"{RCONTROL}", VKeys.VK_RCONTROL},
				{"{LMENU}", VKeys.VK_LMENU},
				{"{RMENU}", VKeys.VK_RMENU},
				{"{PLAY}", VKeys.VK_PLAY},
				{"{ZOOM}", VKeys.VK_ZOOM},
			};

			#endregion

			#region user32.dll native methods

			/// <summary>
			/// Retrieves or sets the value of one of the system-wide parameters. This function can also
			/// update the user profile while setting a parameter.
			/// </summary>
			/// <param name="uiAction">The system-wide parameter to be retrieved or set. The possible values
			/// are organized in the following tables of related parameters.
			/// <see cref="http://msdn.microsoft.com/en-us/library/ms724947%28VS.85%29.aspx"/></param>
			/// <param name="uiParam">A parameter whose usage and format depends on the system parameter being
			/// queried or set. For more information about system-wide parameters, see the uiAction parameter.
			/// If not otherwise indicated, you must specify zero for this parameter.</param>
			/// <param name="pvParam">A parameter whose usage and format depends on the system parameter being
			/// queried or set. For more information about system-wide parameters, see the uiAction parameter.
			/// If not otherwise indicated, you must specify NULL for this parameter.</param>
			/// <param name="fWinIni">If a system parameter is being set, specifies whether the user profile
			/// is to be updated, and if so, whether the WM_SETTINGCHANGE message is to be broadcast to all
			/// top-level windows to notify them of the change. This parameter can be zero if you don't want
			/// to update the user profile or broadcast the WM_SETTINGCHANGE message, or it can be one or
			/// more of the following values.<see cref="<see cref="http://msdn.microsoft.com/en-us/library/ms724947%28VS.85%29.aspx"/>"/>
			/// </param>
			/// <returns>If the function succeeds, the return value is a nonzero value. If the function
			/// fails, the return value is zero. To get extended error information, call GetLastError.
			/// </returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern bool SystemParametersInfo(uint uiAction, uint uiParam, uint pvParam, uint fWinIni);

			/// <summary>
			/// The ShowWindowAsync function sets the show state of a window created by a different thread.
			/// </summary>
			/// <param name="hWnd">Handle to the window.</param>
			/// <param name="nCmdShow">Specifies how the window is to be shown. For a list of possible values,
			/// see the description of the ShowWindow function.</param>
			/// <returns>If the window was previously visible, the return value is nonzero. If the window was
			/// previously hidden, the return value is zero.</returns>
			[DllImport("User32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			private static extern bool ShowWindowAsync(IntPtr hWnd, int nCmdShow);

			/// <summary>
			/// The SetForegroundWindow function puts the thread that created the specified window into the
			/// foreground and activates the window. Keyboard input is directed to the window, and various
			/// visual cues are changed for the user. The system assigns a slightly higher priority to the
			/// thread that created the foreground window than it does to other threads.
			/// </summary>
			/// <param name="hWnd">Handle to the window that should be activated and brought to the foreground.</param>
			/// <returns>If the window was brought to the foreground, the return value is nonzero. If the window
			/// was not brought to the foreground, the return value is zero.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern bool SetForegroundWindow(IntPtr hWnd);

			/// <summary>
			/// The GetWindowText function copies the text of the specified window's title bar
			/// (if it has one) into a buffer. If the specified window is a control, the text of
			/// the control is copied. However, GetWindowText cannot retrieve the text of a control
			/// in another application.
			/// </summary>
			/// <param name="hwnd">Handle to the window or control containing the text.</param>
			/// <param name="lpString">Pointer to the buffer that will receive the text. If the
			/// string is as long or longer than the buffer, the string is truncated and terminated
			/// with a NULL character.</param>
			/// <param name="nMaxCount">Specifies the maximum number of characters to copy to the
			/// buffer, including the NULL character. If the text exceeds this limit, it is truncated.</param>
			/// <returns>If the function succeeds, the return value is the length, in characters, of
			/// the copied string, not including the terminating NULL character. If the window has no
			/// title bar or text, if the title bar is empty, or if the window or control handle is invalid,
			/// the return value is zero. To get extended error information, call GetLastError. This function
			/// cannot retrieve the text of an edit control in another application.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern int GetWindowText(IntPtr hwnd, StringBuilder lpString, int nMaxCount);

			/// <summary>
			/// The PostMessage function places (posts) a message in the message queue associated with the
			/// thread that created the specified window and returns without waiting for the thread to process the message.
			/// To post a message in the message queue associated with a thread, use the PostThreadMessage
			/// function.
			/// </summary>
			/// <param name="hWnd">Microsoft Windows Vista and later. Message posting is subject to
			/// User Interface Privilege Isolation (UIPI). The thread of a process can post messages only
			/// to message queues of threads in processes of lesser or equal integrity level. Handle to the
			/// window whose window procedure is to receive the message. The following values have special
			/// meanings. HWND_BROADCAST - The message is posted to all top-level windows in the system,
			/// including disabled or invisible unowned windows, overlapped windows, and pop-up windows.
			/// The message is not posted to child windows. NULL - The function behaves like a call to
			/// PostThreadMessage with the dwThreadId parameter set to the identifier of the current
			/// thread.</param>
			/// <param name="Msg">Specifies the message to be posted.</param>
			/// <param name="wParam">Specifies additional message-specific information.</param>
			/// <param name="lParam">Specifies additional message-specific information.</param>
			/// <returns></returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr PostMessage(IntPtr hWnd, int Msg, int wParam, int lParam);

			/// <summary>
			/// The FindWindow function retrieves a handle to the top-level window whose class name and
			/// window name match the specified strings. This function does not search child windows.
			/// This function does not perform a case-sensitive search. To search child windows, beginning
			/// with a specified child window, use the FindWindowEx function.
			/// </summary>
			/// <param name="lpClassName">Pointer to a null-terminated string that specifies the class name
			/// or a class atom created by a previous call to the RegisterClass or RegisterClassEx function.
			/// The atom must be in the low-order word of lpClassName; the high-order word must be zero. If
			/// lpClassName points to a string, it specifies the window class name. The class name can be
			/// any name registered with RegisterClass or RegisterClassEx, or any of the predefined
			/// control-class names. If lpClassName is NULL, it finds any window whose title matches the
			/// lpWindowName parameter.</param>
			/// <param name="lpWindowName">Pointer to a null-terminated string that specifies the window
			/// name (the window's title). If this parameter is NULL, all window names match.</param>
			/// <returns>If the function succeeds, the return value is a handle to the window that has
			/// the specified class name and window name. If the function fails, the return value is NULL.
			/// To get extended error information, call GetLastError.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

			/// <summary>
			/// The FindWindowEx function retrieves a handle to a window whose class name and window
			/// name match the specified strings. The function searches child windows, beginning with
			/// the one following the specified child window. This function does not perform a case-sensitive
			/// search.
			/// </summary>
			/// <param name="hwndParent">Handle to the parent window whose child windows are to be searched.
			/// If hwndParent is NULL, the function uses the desktop window as the parent window. The
			/// function searches among windows that are child windows of the desktop. Microsoft Windows 2000
			/// and Windows XP: If hwndParent is HWND_MESSAGE, the function searches all message-only windows. </param>
			/// <param name="hwndChildAfter">Handle to a child window. The search begins with the next child
			/// window in the Z order. The child window must be a direct child window of hwndParent, not
			/// just a descendant window. If hwndChildAfter is NULL, the search begins with the first child
			/// window of hwndParent. Note that if both hwndParent and hwndChildAfter are NULL, the function
			/// searches all top-level and message-only windows.</param>
			/// <param name="lpszClass">Pointer to a null-terminated string that specifies the class name
			/// or a class atom created by a previous call to the RegisterClass or RegisterClassEx function.
			/// The atom must be placed in the low-order word of lpszClass; the high-order word must be zero.
			/// If lpszClass is a string, it specifies the window class name. The class name can be any name
			/// registered with RegisterClass or RegisterClassEx, or any of the predefined control-class names,
			/// or it can be MAKEINTATOM(0x800). In this latter case, 0x8000 is the atom for a menu class.
			/// For more information, see the Remarks section of this topic.</param>
			/// <param name="lpszWindow">Pointer to a null-terminated string that specifies the window name
			/// (the window's title). If this parameter is NULL, all window names match.</param>
			/// <returns>If the function succeeds, the return value is a handle to the window that has the
			/// specified class and window names. If the function fails, the return value is NULL. To get
			/// extended error information, call GetLastError.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr FindWindowEx(IntPtr hwndParent, IntPtr hwndChildAfter, string lpszClass, string lpszWindow);

			/// <summary>
			/// Sends the specified message to a window or windows. The SendMessage function calls the
			/// window procedure for the specified window and does not return until the window procedure has
			/// processed the message. To send a message and return immediately, use the SendMessageCallback
			/// or SendNotifyMessage function. To post a message to a thread's message queue and return
			/// immediately, use the PostMessage or PostThreadMessage function.
			/// </summary>
			/// <param name="hWnd">Handle to the window whose window procedure will receive the message. If this parameter is HWND_BROADCAST, the message is sent to all top-level windows in the system, including disabled or invisible unowned windows, overlapped windows, and pop-up windows; but the message is not sent to child windows.
			/// Microsoft Windows Vista and later. Message sending is subject to User Interface Privilege
			/// Isolation (UIPI). The thread of a process can send messages only to message queues of
			/// threads in processes of lesser or equal integrity level.</param>
			/// <param name="Msg">Specifies the message to be sent.</param>
			/// <param name="wParam">Specifies additional message-specific information.</param>
			/// <param name="lParam">Specifies additional message-specific information.</param>
			/// <returns>The return value specifies the result of the message processing; it depends on
			/// the message sent.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr SendMessage(IntPtr hWnd, int Msg, int wParam, int lParam);

			/// <summary>
			/// The SetWindowText function changes the text of the specified window's title bar (if it has one).
			/// If the specified window is a control, the text of the control is changed.
			/// However, SetWindowText cannot change the text of a control in another application.
			/// </summary>
			/// <param name="hWnd">Handle to the window or control whose text is to be changed.</param>
			/// <param name="lpString">Pointer to a null-terminated string to be used as the new title
			/// or control text. </param>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern void SetWindowText(IntPtr hWnd, string lpString);

			/// <summary>
			/// The GetDesktopWindow function returns a handle to the desktop window. The desktop window
			/// covers the entire screen. The desktop window is the area on top of which other windows are
			/// painted.
			/// </summary>
			/// <returns>The return value is a handle to the desktop window.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr GetDesktopWindow();
			
			/// <summary>
			/// The GetWindowDC function retrieves the device context (DC) for the entire window,
			/// including title bar, menus, and scroll bars. A window device context permits painting
			/// anywhere in a window, because the origin of the device context is the upper-left corner
			/// of the window instead of the client area.
			/// </summary>
			/// <param name="hWnd">A handle to the window with a device context that is to be retrieved.
			/// If this value is NULL, GetWindowDC retrieves the device context for the entire screen.
			/// If this parameter is NULL, GetWindowDC retrieves the device context for the primary display
			/// monitor. To get the device context for other display monitors, use the EnumDisplayMonitors
			/// and CreateDC functions.
			/// </param>
			/// <returns>If the function succeeds, the return value is a handle to a device context for the
			/// specified window. If the function fails, the return value is NULL, indicating an error or
			/// an invalid hWnd parameter.</returns>
			[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
			public static extern IntPtr GetWindowDC(IntPtr hWnd);
			#endregion

			#region public methods

			/// <summary>
			/// Returns the window handle of specified window name.
			/// </summary>
			/// <param name="windowTitle">Name of the window.</param>
			/// <returns>The window handle.</returns>
			public static IntPtr FindWindowByTitle(string windowTitle)
			{
				return
					Win32.User.FindWindow(null, windowTitle);
			}

			/// <summary>
			/// Returns the window handle of specified window class.
			/// </summary>
			/// <param name="className">Name of the class.</param>
			/// <returns>The window handle.</returns>
			public static IntPtr FindWindowByClassName(string className)
			{
				return
					Win32.User.FindWindow(className, null);
			}

			/// <summary>
			/// Focuses the console window.
			/// </summary>
			public static void FocusConsoleWindow()
			{
				IntPtr hWnd = FindWindowByClassName(User.ConsoleClassName);
				if (hWnd != null)
				{
					Win32.User.SetForegroundWindow(hWnd);
				}
			}

			/// <summary>
			/// Send a keyboard input to a window.
			/// </summary>
			/// <param name="windowHandle">The window handle.</param>
			/// <param name="className">Name of the control class in the window.</param>
			/// <param name="inputStr">The input string.</param>
			public static void KeyboardInput(IntPtr windowHandle, string className, string inputStr)
			{
				IntPtr controlHandle =
					Win32.User.FindWindowEx(windowHandle, IntPtr.Zero, className, null);

				Win32.User.KeyboardInput(controlHandle, inputStr);
			}

			/// <summary>
			/// Send keyboard input to a window.
			/// </summary>
			/// <param name="windowHandle">The window handle.</param>
			/// <param name="inputStr">The input string.</param>
			public static void KeyboardInput(IntPtr windowHandle, string inputStr)
			{
				bool specialToken = false;
				string tokenStr = string.Empty;

				foreach (char ch in inputStr.ToUpper())
				{
					if (ch == '{')
					{
						specialToken = true;
						tokenStr = ch.ToString();
						continue;
					}
					else if (ch == '}' && specialToken)
					{
						tokenStr += ch.ToString();
						if (Keys.ContainsKey(tokenStr))
						{
							Win32.User.PostMessage(windowHandle, (int)WMessages.WM_KEYDOWN, (int)Keys[tokenStr], 0);
						}
						else
						{
							foreach (char tokenCh in tokenStr)
							{
								string keyStr = tokenCh.ToString();
								if (Keys.ContainsKey(keyStr))
								{
									Win32.User.PostMessage(windowHandle, (int)WMessages.WM_KEYDOWN, (int)Keys[keyStr], 0);
								}
							}
						}
						tokenStr = string.Empty;
						specialToken = false;
					}
					else
					{
						if (specialToken)
							tokenStr += ch.ToString();
						else
						{
							string keyStr = ch.ToString();
							if (Keys.ContainsKey(keyStr))
							{
								Win32.User.PostMessage(windowHandle, (int)WMessages.WM_KEYDOWN, (int)Keys[keyStr], 0);
							}
						}
					}
				}

				if (!string.IsNullOrEmpty(tokenStr))
				{
					foreach (char tokenCh in tokenStr)
					{
						string keyStr = tokenCh.ToString();
						if (Keys.ContainsKey(keyStr))
						{
							Win32.User.PostMessage(windowHandle, (int)WMessages.WM_KEYDOWN, (int)Keys[keyStr], 0);
						}
					}
				}
			}

			/// <summary>
			/// Performs a mouse click in a specific window location.
			/// </summary>
			/// <param name="windowTitle">The window title string.</param>
			/// <param name="button">The name of the button to click.</param>
			/// <param name="x">The mouse x-coordinate.</param>
			/// <param name="y">The mouse y-cooridnate.</param>
			/// <param name="performDoubleClick">if set to <c>true</c> simulate a mouse double click.</param>
			public static void ClickWindow(string windowTitle, string button, int x, int y, bool performDoubleClick)
			{
				IntPtr hWnd = Win32.User.FindWindow(null, windowTitle);
				int LParam = Win32.User.MakeLParam(x, y);

				int btnDown = 0;
				int btnUp = 0;

				if (button == "left")
				{
					btnDown = (int)WMessages.WM_LBUTTONDOWN;
					btnUp = (int)WMessages.WM_LBUTTONUP;
				}

				if (button == "right")
				{
					btnDown = (int)WMessages.WM_RBUTTONDOWN;
					btnUp = (int)WMessages.WM_RBUTTONUP;
				}

				if (performDoubleClick)
				{
					Win32.User.PostMessage(hWnd, btnDown, 0, LParam);
					Win32.User.PostMessage(hWnd, btnUp, 0, LParam);
					Win32.User.PostMessage(hWnd, btnDown, 0, LParam);
					Win32.User.PostMessage(hWnd, btnUp, 0, LParam);
				}
				else
				{
					Win32.User.PostMessage(hWnd, btnDown, 0, LParam);
					Win32.User.PostMessage(hWnd, btnUp, 0, LParam);
				}
			}

			#endregion

			#region Private methods


			/// <summary>
			/// Pack low and high word into to word.
			/// </summary>
			/// <param name="LoWord">The low word.</param>
			/// <param name="HiWord">The high word.</param>
			/// <returns>The packed integer.</returns>
			public static int MakeLParam(int LoWord, int HiWord)
			{
				return ((HiWord << 16) | (LoWord & 0xffff));
			}

			#endregion
		}
	}
}





namespace ScreenshotCaptureWithMouse.ScreenCapture
{
	class Win32Stuff
	{

		#region Class Variables
		
		public const int SM_CXSCREEN = 0;
		public const int SM_CYSCREEN = 1;

		public const Int32 CURSOR_SHOWING = 0x00000001;

		[StructLayout(LayoutKind.Sequential)]
		public struct ICONINFO
		{
			public bool fIcon;         // Specifies whether this structure defines an icon or a cursor. A value of TRUE specifies
			public Int32 xHotspot;     // Specifies the x-coordinate of a cursor's hot spot. If this structure defines an icon, the hot
			public Int32 yHotspot;     // Specifies the y-coordinate of the cursor's hot spot. If this structure defines an icon, the hot
			public IntPtr hbmMask;     // (HBITMAP) Specifies the icon bitmask bitmap. If this structure defines a black and white icon,
			public IntPtr hbmColor;    // (HBITMAP) Handle to the icon color bitmap. This member can be optional if this
		}
		[StructLayout(LayoutKind.Sequential)]
		public struct POINT
		{
			public Int32 x;
			public Int32 y;
		}

		[StructLayout(LayoutKind.Sequential)]
		public struct CURSORINFO
		{
			public Int32 cbSize;        // Specifies the size, in bytes, of the structure.
			public Int32 flags;         // Specifies the cursor state. This parameter can be one of the following values:
			public IntPtr hCursor;          // Handle to the cursor.
			public POINT ptScreenPos;       // A POINT structure that receives the screen coordinates of the cursor.
		}

		#endregion


		#region Class Functions

		[DllImport("user32.dll", EntryPoint = "GetDesktopWindow")]
		public static extern IntPtr GetDesktopWindow();

		[DllImport("user32.dll", EntryPoint = "GetDC")]
		public static extern IntPtr GetDC(IntPtr ptr);

		[DllImport("user32.dll", EntryPoint = "GetWindowDC")]
		public static extern IntPtr GetWindowDC(IntPtr ptr);
		
		[DllImport("user32.dll", EntryPoint = "GetSystemMetrics")]
		public static extern int GetSystemMetrics(int abc);

		[DllImport("user32.dll", EntryPoint = "GetWindowDC")]
		public static extern IntPtr GetWindowDC(Int32 ptr);

		[DllImport("user32.dll", EntryPoint = "ReleaseDC")]
		public static extern IntPtr ReleaseDC(IntPtr hWnd, IntPtr hDc);


		[DllImport("user32.dll", EntryPoint = "GetCursorInfo")]
		public static extern bool GetCursorInfo(out CURSORINFO pci);

		[DllImport("user32.dll", EntryPoint = "CopyIcon")]
		public static extern IntPtr CopyIcon(IntPtr hIcon);

		[DllImport("user32.dll", EntryPoint = "GetIconInfo")]
		public static extern bool GetIconInfo(IntPtr hIcon, out ICONINFO piconinfo);

		[DllImport("user32.dll")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool DestroyIcon(IntPtr hIcon);
		

		#endregion
	}
}
